@@ -1,21 +1,23 @@
 import unittest
 
-from django.conf import settings
 from django.core.checks import Error, Warning
 from django.core.checks.model_checks import _check_lazy_references
-from django.core.exceptions import ImproperlyConfigured
 from django.db import connection, connections, models
 from django.db.models.functions import Lower
 from django.db.models.signals import post_init
-from django.test import SimpleTestCase
+from django.test import SimpleTestCase, TestCase, skipUnlessDBFeature
 from django.test.utils import isolate_apps, override_settings, register_lookup
 
 
+class EmptyRouter:
+    pass
+
+
 def get_max_column_name_length():
     allowed_len = None
     db_alias = None
 
-    for db in settings.DATABASES:
+    for db in ('default', 'other'):
         connection = connections[db]
         max_name_length = connection.ops.max_name_length()
         if max_name_length is not None and not connection.features.truncates_names:
@@ -232,7 +234,7 @@ class UniqueTogetherTests(SimpleTestCase):
 
 
 @isolate_apps('invalid_models_tests')
-class IndexesTests(SimpleTestCase):
+class IndexesTests(TestCase):
 
     def test_pointing_to_missing_field(self):
         class Model(models.Model):
@@ -329,9 +331,54 @@ class IndexesTests(SimpleTestCase):
             ),
         ])
 
+    def test_index_with_condition(self):
+        class Model(models.Model):
+            age = models.IntegerField()
+
+            class Meta:
+                indexes = [
+                    models.Index(
+                        fields=['age'],
+                        name='index_age_gte_10',
+                        condition=models.Q(age__gte=10),
+                    ),
+                ]
+
+        errors = Model.check(databases=self.databases)
+        expected = [] if connection.features.supports_partial_indexes else [
+            Warning(
+                '%s does not support indexes with conditions.'
+                % connection.display_name,
+                hint=(
+                    "Conditions will be ignored. Silence this warning if you "
+                    "don't care about it."
+                ),
+                obj=Model,
+                id='models.W037',
+            )
+        ]
+        self.assertEqual(errors, expected)
+
+    def test_index_with_condition_required_db_features(self):
+        class Model(models.Model):
+            age = models.IntegerField()
+
+            class Meta:
+                required_db_features = {'supports_partial_indexes'}
+                indexes = [
+                    models.Index(
+                        fields=['age'],
+                        name='index_age_gte_10',
+                        condition=models.Q(age__gte=10),
+                    ),
+                ]
+
+        self.assertEqual(Model.check(databases=self.databases), [])
+
 
 @isolate_apps('invalid_models_tests')
-class FieldNamesTests(SimpleTestCase):
+class FieldNamesTests(TestCase):
+    databases = {'default', 'other'}
 
     def test_ending_with_underscore(self):
         class Model(models.Model):
@@ -359,8 +406,6 @@ class FieldNamesTests(SimpleTestCase):
         #13711 -- Model check for long M2M column names when database has
         column name length limits.
         """
-        allowed_len, db_alias = get_max_column_name_length()
-
         # A model with very long name which will be used to set relations to.
         class VeryLongModelNamezzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz(models.Model):
             title = models.CharField(max_length=11)
@@ -405,7 +450,7 @@ class FieldNamesTests(SimpleTestCase):
             db_column=long_field_name
         ).contribute_to_class(m2mcomplex, long_field_name)
 
-        errors = ModelWithLongField.check()
+        errors = ModelWithLongField.check(databases=('default', 'other'))
 
         # First error because of M2M field set on the model with long name.
         m2m_long_name = "verylongmodelnamezzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz_id"
@@ -443,6 +488,9 @@ class FieldNamesTests(SimpleTestCase):
         )
 
         self.assertEqual(errors, expected)
+        # Check for long column names is called only for specified database
+        # aliases.
+        self.assertEqual(ModelWithLongField.check(databases=None), [])
 
     @unittest.skipIf(max_column_name_length is None, "The database doesn't have a column name length limit.")
     def test_local_field_long_column_name(self):
@@ -450,8 +498,6 @@ class FieldNamesTests(SimpleTestCase):
         #13711 -- Model check for long column names
         when database does not support long names.
         """
-        allowed_len, db_alias = get_max_column_name_length()
-
         class ModelWithLongField(models.Model):
             title = models.CharField(max_length=11)
 
@@ -459,7 +505,7 @@ class FieldNamesTests(SimpleTestCase):
         long_field_name2 = 'b' * (self.max_column_name_length + 1)
         models.CharField(max_length=11).contribute_to_class(ModelWithLongField, long_field_name)
         models.CharField(max_length=11, db_column='vlmn').contribute_to_class(ModelWithLongField, long_field_name2)
-        self.assertEqual(ModelWithLongField.check(), [
+        self.assertEqual(ModelWithLongField.check(databases=('default', 'other')), [
             Error(
                 'Autogenerated column name too long for field "%s". '
                 'Maximum length is "%s" for database "%s".'
@@ -469,6 +515,9 @@ class FieldNamesTests(SimpleTestCase):
                 id='models.E018',
             )
         ])
+        # Check for long column names is called only for specified database
+        # aliases.
+        self.assertEqual(ModelWithLongField.check(databases=None), [])
 
     def test_including_separator(self):
         class Model(models.Model):
@@ -1015,14 +1064,24 @@ class OtherModelTests(SimpleTestCase):
 
         self.assertEqual(ShippingMethod.check(), [])
 
-    def test_missing_parent_link(self):
-        msg = 'Add parent_link=True to invalid_models_tests.ParkingLot.parent.'
-        with self.assertRaisesMessage(ImproperlyConfigured, msg):
-            class Place(models.Model):
-                pass
+    def test_onetoone_with_parent_model(self):
+        class Place(models.Model):
+            pass
 
-            class ParkingLot(Place):
-                parent = models.OneToOneField(Place, models.CASCADE)
+        class ParkingLot(Place):
+            other_place = models.OneToOneField(Place, models.CASCADE, related_name='other_parking')
+
+        self.assertEqual(ParkingLot.check(), [])
+
+    def test_onetoone_with_explicit_parent_link_parent_model(self):
+        class Place(models.Model):
+            pass
+
+        class ParkingLot(Place):
+            place = models.OneToOneField(Place, models.CASCADE, parent_link=True, primary_key=True)
+            other_place = models.OneToOneField(Place, models.CASCADE, related_name='other_parking')
+
+        self.assertEqual(ParkingLot.check(), [])
 
     def test_m2m_table_name_clash(self):
         class Foo(models.Model):
@@ -1044,6 +1103,32 @@ class OtherModelTests(SimpleTestCase):
             )
         ])
 
+    @override_settings(DATABASE_ROUTERS=['invalid_models_tests.test_models.EmptyRouter'])
+    def test_m2m_table_name_clash_database_routers_installed(self):
+        class Foo(models.Model):
+            bar = models.ManyToManyField('Bar', db_table='myapp_bar')
+
+            class Meta:
+                db_table = 'myapp_foo'
+
+        class Bar(models.Model):
+            class Meta:
+                db_table = 'myapp_bar'
+
+        self.assertEqual(Foo.check(), [
+            Warning(
+                "The field's intermediary table 'myapp_bar' clashes with the "
+                "table name of 'invalid_models_tests.Bar'.",
+                obj=Foo._meta.get_field('bar'),
+                hint=(
+                    "You have configured settings.DATABASE_ROUTERS. Verify "
+                    "that the table of 'invalid_models_tests.Bar' is "
+                    "correctly routed to a separate database."
+                ),
+                id='fields.W344',
+            ),
+        ])
+
     def test_m2m_field_table_name_clash(self):
         class Foo(models.Model):
             pass
@@ -1069,6 +1154,32 @@ class OtherModelTests(SimpleTestCase):
             )
         ])
 
+    @override_settings(DATABASE_ROUTERS=['invalid_models_tests.test_models.EmptyRouter'])
+    def test_m2m_field_table_name_clash_database_routers_installed(self):
+        class Foo(models.Model):
+            pass
+
+        class Bar(models.Model):
+            foos = models.ManyToManyField(Foo, db_table='clash')
+
+        class Baz(models.Model):
+            foos = models.ManyToManyField(Foo, db_table='clash')
+
+        self.assertEqual(Bar.check() + Baz.check(), [
+            Warning(
+                "The field's intermediary table 'clash' clashes with the "
+                "table name of 'invalid_models_tests.%s.foos'."
+                % clashing_model,
+                obj=model_cls._meta.get_field('foos'),
+                hint=(
+                    "You have configured settings.DATABASE_ROUTERS. Verify "
+                    "that the table of 'invalid_models_tests.%s.foos' is "
+                    "correctly routed to a separate database." % clashing_model
+                ),
+                id='fields.W344',
+            ) for model_cls, clashing_model in [(Bar, 'Baz'), (Baz, 'Bar')]
+        ])
+
     def test_m2m_autogenerated_table_name_clash(self):
         class Foo(models.Model):
             class Meta:
@@ -1090,6 +1201,33 @@ class OtherModelTests(SimpleTestCase):
             )
         ])
 
+    @override_settings(DATABASE_ROUTERS=['invalid_models_tests.test_models.EmptyRouter'])
+    def test_m2m_autogenerated_table_name_clash_database_routers_installed(self):
+        class Foo(models.Model):
+            class Meta:
+                db_table = 'bar_foos'
+
+        class Bar(models.Model):
+            # The autogenerated db_table is bar_foos.
+            foos = models.ManyToManyField(Foo)
+
+            class Meta:
+                db_table = 'bar'
+
+        self.assertEqual(Bar.check(), [
+            Warning(
+                "The field's intermediary table 'bar_foos' clashes with the "
+                "table name of 'invalid_models_tests.Foo'.",
+                obj=Bar._meta.get_field('foos'),
+                hint=(
+                    "You have configured settings.DATABASE_ROUTERS. Verify "
+                    "that the table of 'invalid_models_tests.Foo' is "
+                    "correctly routed to a separate database."
+                ),
+                id='fields.W344',
+            ),
+        ])
+
     def test_m2m_unmanaged_shadow_models_not_checked(self):
         class A1(models.Model):
             pass
@@ -1212,7 +1350,41 @@ class OtherModelTests(SimpleTestCase):
 
 
 @isolate_apps('invalid_models_tests')
-class ConstraintsTests(SimpleTestCase):
+class JSONFieldTests(TestCase):
+    @skipUnlessDBFeature('supports_json_field')
+    def test_ordering_pointing_to_json_field_value(self):
+        class Model(models.Model):
+            field = models.JSONField()
+
+            class Meta:
+                ordering = ['field__value']
+
+        self.assertEqual(Model.check(databases=self.databases), [])
+
+    def test_check_jsonfield(self):
+        class Model(models.Model):
+            field = models.JSONField()
+
+        error = Error(
+            '%s does not support JSONFields.' % connection.display_name,
+            obj=Model,
+            id='fields.E180',
+        )
+        expected = [] if connection.features.supports_json_field else [error]
+        self.assertEqual(Model.check(databases=self.databases), expected)
+
+    def test_check_jsonfield_required_db_features(self):
+        class Model(models.Model):
+            field = models.JSONField()
+
+            class Meta:
+                required_db_features = {'supports_json_field'}
+
+        self.assertEqual(Model.check(databases=self.databases), [])
+
+
+@isolate_apps('invalid_models_tests')
+class ConstraintsTests(TestCase):
     def test_check_constraints(self):
         class Model(models.Model):
             age = models.IntegerField()
@@ -1220,7 +1392,7 @@ class ConstraintsTests(SimpleTestCase):
             class Meta:
                 constraints = [models.CheckConstraint(check=models.Q(age__gte=18), name='is_adult')]
 
-        errors = Model.check()
+        errors = Model.check(databases=self.databases)
         warn = Warning(
             '%s does not support check constraints.' % connection.display_name,
             hint=(
@@ -1230,7 +1402,7 @@ class ConstraintsTests(SimpleTestCase):
             obj=Model,
             id='models.W027',
         )
-        expected = [] if connection.features.supports_table_check_constraints else [warn, warn]
+        expected = [] if connection.features.supports_table_check_constraints else [warn]
         self.assertCountEqual(errors, expected)
 
     def test_check_constraints_required_db_features(self):
@@ -1240,5 +1412,92 @@ class ConstraintsTests(SimpleTestCase):
             class Meta:
                 required_db_features = {'supports_table_check_constraints'}
                 constraints = [models.CheckConstraint(check=models.Q(age__gte=18), name='is_adult')]
+        self.assertEqual(Model.check(databases=self.databases), [])
 
-        self.assertEqual(Model.check(), [])
+    def test_unique_constraint_with_condition(self):
+        class Model(models.Model):
+            age = models.IntegerField()
+
+            class Meta:
+                constraints = [
+                    models.UniqueConstraint(
+                        fields=['age'],
+                        name='unique_age_gte_100',
+                        condition=models.Q(age__gte=100),
+                    ),
+                ]
+
+        errors = Model.check(databases=self.databases)
+        expected = [] if connection.features.supports_partial_indexes else [
+            Warning(
+                '%s does not support unique constraints with conditions.'
+                % connection.display_name,
+                hint=(
+                    "A constraint won't be created. Silence this warning if "
+                    "you don't care about it."
+                ),
+                obj=Model,
+                id='models.W036',
+            ),
+        ]
+        self.assertEqual(errors, expected)
+
+    def test_unique_constraint_with_condition_required_db_features(self):
+        class Model(models.Model):
+            age = models.IntegerField()
+
+            class Meta:
+                required_db_features = {'supports_partial_indexes'}
+                constraints = [
+                    models.UniqueConstraint(
+                        fields=['age'],
+                        name='unique_age_gte_100',
+                        condition=models.Q(age__gte=100),
+                    ),
+                ]
+
+        self.assertEqual(Model.check(databases=self.databases), [])
+
+    def test_deferrable_unique_constraint(self):
+        class Model(models.Model):
+            age = models.IntegerField()
+
+            class Meta:
+                constraints = [
+                    models.UniqueConstraint(
+                        fields=['age'],
+                        name='unique_age_deferrable',
+                        deferrable=models.Deferrable.DEFERRED,
+                    ),
+                ]
+
+        errors = Model.check(databases=self.databases)
+        expected = [] if connection.features.supports_deferrable_unique_constraints else [
+            Warning(
+                '%s does not support deferrable unique constraints.'
+                % connection.display_name,
+                hint=(
+                    "A constraint won't be created. Silence this warning if "
+                    "you don't care about it."
+                ),
+                obj=Model,
+                id='models.W038',
+            ),
+        ]
+        self.assertEqual(errors, expected)
+
+    def test_deferrable_unique_constraint_required_db_features(self):
+        class Model(models.Model):
+            age = models.IntegerField()
+
+            class Meta:
+                required_db_features = {'supports_deferrable_unique_constraints'}
+                constraints = [
+                    models.UniqueConstraint(
+                        fields=['age'],
+                        name='unique_age_deferrable',
+                        deferrable=models.Deferrable.IMMEDIATE,
+                    ),
+                ]
+
+        self.assertEqual(Model.check(databases=self.databases), [])
