@@ -3,7 +3,9 @@ from django.contrib.contenttypes.fields import (
     GenericForeignKey, GenericRelation,
 )
 from django.core.exceptions import FieldDoesNotExist
-from django.db.models.fields import CharField, Field, related
+from django.db.models import (
+    CharField, Field, ForeignObjectRel, ManyToManyField,
+)
 from django.db.models.options import EMPTY_RELATION_TREE, IMMUTABLE_WARNING
 from django.test import SimpleTestCase
 
@@ -175,17 +177,17 @@ class GetFieldByNameTests(OptionsBaseTests):
     def test_get_m2m_field(self):
         field_info = self._details(Person, Person._meta.get_field('m2m_base'))
         self.assertEqual(field_info[1:], (BasePerson, True, True))
-        self.assertIsInstance(field_info[0], related.ManyToManyField)
+        self.assertIsInstance(field_info[0], ManyToManyField)
 
     def test_get_related_object(self):
         field_info = self._details(Person, Person._meta.get_field('relating_baseperson'))
         self.assertEqual(field_info[1:], (BasePerson, False, False))
-        self.assertIsInstance(field_info[0], related.ForeignObjectRel)
+        self.assertIsInstance(field_info[0], ForeignObjectRel)
 
     def test_get_related_m2m(self):
         field_info = self._details(Person, Person._meta.get_field('relating_people'))
         self.assertEqual(field_info[1:], (None, False, True))
-        self.assertIsInstance(field_info[0], related.ForeignObjectRel)
+        self.assertIsInstance(field_info[0], ForeignObjectRel)
 
     def test_get_generic_relation(self):
         field_info = self._details(Person, Person._meta.get_field('generic_relation_base'))
